# SPDX-FileCopyrightText: Copyright (c) 2024-25, NVIDIA CORPORATION & AFFILIATES.
# All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
Benchmark commands for measuring actor-stage throughput.
"""

from .__main__ import app

__all__ = ["app"]
