# SPDX-FileCopyrightText: Copyright (c) 2024-25, NVIDIA CORPORATION & AFFILIATES.
# All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
Ingestion runmodes.

Each runmode provides a concrete `Ingestor` implementation with the same public
method surface defined by `nemo_retriever.ingestor.Ingestor`.
"""

from __future__ import annotations

from .batch import BatchIngestor
from .fused import FusedIngestor
from .inprocess import InProcessIngestor
from .online import OnlineIngestor

__all__ = [
    "BatchIngestor",
    "FusedIngestor",
    "InProcessIngestor",
    "OnlineIngestor",
]
