# SPDX-FileCopyrightText: Copyright (c) 2024-25, NVIDIA CORPORATION & AFFILIATES.
# All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from nemo_retriever.adapters.service.app import RetrieverAPIDeployment, app

__all__ = ["RetrieverAPIDeployment", "app"]
